import express from 'express';
import { IdentityManager } from '..';
import { MetricsStorage } from '../metrics/metricsStorage';
import { IAppOptions } from '../model/appOptions';
import { IConfiguration } from '../model/configuration';
export declare class BacktraceService<T extends {}> {
    private readonly name;
    private readonly port;
    private readonly _opts?;
    /**
     * Create and startup express application with basic options.
     * By using this method, application will apply all
     * Backtrace service rules
     * @param name service name (required by service layer)
     * @param port service port (required by service layer)
     * @param opts optional application options
     * @param app  optional express app instance
     * By using this parameter you can create express app in different
     * framework and still be able to use Backtrace integrations
     */
    static bootstrap<T extends {}>(name: string, port: number, opts?: IAppOptions, app?: express.Express): BacktraceService<T>;
    static readConfiguration<T = {}>(name: string, port: number, path?: string, ignoreDescriptor?: boolean): IConfiguration<T>;
    /**
     * Setup popular Backtrace middlewares
     * @param app express app
     */
    static setupMiddlewares(app: express.Express): void;
    /**
     * Setup security middlewares
     * @param app express app
     */
    static setupAuth(app: express.Express): void;
    /**
     * Setup service layer integration
     * @param app express app
     */
    static setupServiceLayer(app: express.Express, name?: string, port?: number, coronerdCallback?: (param: string) => void): void;
    private static _service;
    readonly configuration: IConfiguration<T>;
    readonly coronerdAddress: Set<string>;
    identityManager: IdentityManager;
    readonly app: express.Express;
    readonly metrics: MetricsStorage;
    private readonly _logger;
    private _descriptor;
    constructor(name: string, port: number, _opts?: IAppOptions | undefined, app?: express.Express);
    start(): void;
    private prepareServiceLayer;
}
