const fs = require('fs');
const { ServiceConfiguration } = require('backtrace-service');
const confPath = require('./confPath');

/**
 *  Function to normalize backtrace-saml config to work with the shape that
 *  backtrace-service expects.
 */
function normalize(config) {
  if (process.env.NODE_ENV === 'production') {
    if (config.bind && config.bind.ssl) {
      config.ssl = {
        key: fs.readFileSync(config.bind.ssl.key, 'utf-8'),
        cert: fs.readFileSync(config.bind.ssl.certificate, 'utf-8'),
      };
    }
    if (config.enableBacktraceReporting) {
      config.backtrace = Object.assign(
        {
          endpoint:
            'https://submit.backtrace.io/backtrace/11524298f231d27194b210052ad6c9ce64d1aaf10bd5994fad4a959940433f26/json',
          token:
            '11524298f231d27194b210052ad6c9ce64d1aaf10bd5994fad4a959940433f26',
        },
        config.backtrace
      );
    }
  }
}

// Gets port to establish service + service descriptor if not already created
// NOTE: legacy systems declare port in config file
// AND config port value can be an Array of ports
// saml.backtrace.io operates like this for backwards compat
// undefined => array(int);
function getPorts() {
  const DEFAULT_PORT = 6446;
  const serviceConfiguration = new ServiceConfiguration();
  const configuration = serviceConfiguration.loadConfiguration(confPath);

  if (configuration.isProduction) {
    if (configuration.bind && configuration.bind.port) {
      const { port } = configuration.bind;
      return Array.isArray(port) ? port : [port];
    } else {
      return [DEFAULT_PORT];
    }
  } else {
    return [3000];
  }
}

module.exports = {
  normalize,
  getPorts,
};
